import java.awt.*;
import java.io.* ;
import java.util.* ;


class Clock
{
  StatusDisplay statusDisplay;
  int max_timefac ;		//  maximum time compression factor
  int timefac ;		        //  current time compression factor
    //  time in ms that train thread had been suspended
  int suspended_ms ;
  int currms ;		        //  current time in milliseconds
  int currDisplayMs;            //  current display time in milliseconds
  int prevDisplayMinutes;
  int old_train_ms ;		//  time before current repaint
  boolean newClock;

    //  wait for 30 seconds at each station
  static final int STATION_SLEEP = 30000 ;
    //  1 minute in milliseconds
  static final int MS_IN_MINUTE = 60000 ;

  Clock(int maxfac, int tfac, int startScheduleTime) 
  {
      // maximum time compression factor
    timefac = max_timefac = maxfac ;
    statusDisplay = new StatusDisplay();
    update(true, startScheduleTime);
  }

  void update(boolean do_reset, int startScheduleTime)
  {
    String timeTxt;

    if (do_reset)
    {
        // current time in milliseconds
      currms = (int) System.currentTimeMillis() ;
      old_train_ms = currms ;
      currDisplayMs = startScheduleTime;
      prevDisplayMinutes = currDisplayMs/MS_IN_MINUTE;
    }
    newClock = true;
    timeTxt = statusDisplay.convertTime(currDisplayMs/MS_IN_MINUTE, timefac);
    statusDisplay.setText(timeTxt);
  }


  void paint(Graphics g, boolean redrawBackground, boolean endBART)
  {
    if (!newClock)
      statusDisplay.paint(g, redrawBackground, endBART);
    else
    {
      statusDisplay.paint(g, true, endBART);
      newClock = false;
    }
  }


      //  real-time or speeded-up mode	
  void setRealMode(boolean real_choice)
  {
    if (real_choice)
      timefac = 1 ;
    else
      timefac = max_timefac ;
  }


  int getRealMode()
  {
    return(timefac);
  }


  int getDailyTimeinMS(String timeStr)
  {
    String hourStr;
    String minuteStr;
    char am_or_pm;
    boolean isAm;
    int hourInt;
    int minuteInt;

    int colonPos = timeStr.indexOf(':');
    if (colonPos == -1)
      return (-1);
    am_or_pm = timeStr.charAt(colonPos+3);
    if (am_or_pm == 'a')
      isAm = true;
    else if (am_or_pm == 'p')
      isAm = false;
    else
      return (-1);
    hourStr = timeStr.substring(0,colonPos);
    hourInt = Integer.parseInt(hourStr);
    if ((hourInt < 0) || (hourInt > 12))
      return (-1);
    minuteStr = timeStr.substring(colonPos+1,colonPos+3);
    minuteInt = Integer.parseInt(minuteStr);
    if ((minuteInt < 0) || (minuteInt > 59))
      return (-1);
    if (!isAm)
    {
      if (hourInt != 12)
        hourInt += 12;
    }
    else
    {
        //  kludge
      if ((hourInt == 1) && (minuteInt < 23))
        hourInt += 24 ;
      else if (hourInt == 12)
        hourInt += 12;
    }
    return (hourInt*60*MS_IN_MINUTE + minuteInt*MS_IN_MINUTE);
  }


  int timeElapsed()
  {
        //  difference in ms between last update and this
    int diff_ms ;

    currms = (int) System.currentTimeMillis() ;
      //  difference between this and last time through
      //  loop (repaint was called in the intervening time)
    diff_ms = (currms - old_train_ms) * timefac ;
//    System.out.println(currms - old_train_ms);
    currDisplayMs += diff_ms ;
      //  if rounded off to next minute, redraw time
    if ((currDisplayMs/MS_IN_MINUTE) != prevDisplayMinutes)
    {
      String timeTxt = statusDisplay.convertTime(currDisplayMs/MS_IN_MINUTE, timefac);
      statusDisplay.setText(timeTxt);
      prevDisplayMinutes = currDisplayMs/MS_IN_MINUTE;
    }
    return(diff_ms);
  }

  void adjustRate(int diff_ms)
  {
    int slow_down = 0 ;	//  slow the simulation down
    int speed_up = 0 ;	//  speed it up
    int prev_timefac = timefac;

    old_train_ms = currms ;
      // magic heuristic numbers
    if ((diff_ms/timefac) > 100)
    {
      slow_down++ ;
      speed_up = 0 ;
    }
    else
      slow_down = 0 ;
        //  if system getting behind, slow simulation down
    if ((slow_down > 2) && (timefac > 10))
    {
      timefac -= 10 ;
      slow_down = 0 ;
    }
    if (((diff_ms/timefac) < 40) && (timefac < max_timefac))
      speed_up++ ;
    else
      speed_up = 0 ;
    if (timefac == 1)
      speed_up = 0 ;
        //  relatively safe to speed simulation up
    if (speed_up > 47)
    {
      timefac++ ;
      speed_up = 0 ;
    }
    if (prev_timefac != timefac)
    {
      String timeTxt = statusDisplay.convertTime(currDisplayMs/MS_IN_MINUTE, timefac);
      statusDisplay.setText(timeTxt);
    }
  }

  void suspendTime()
  {
      //  time at suspension
    suspended_ms = (int) System.currentTimeMillis() ;
  }


  void reStartTime()
  {
      //  account for time that trains were suspended
    currms = (int) System.currentTimeMillis() ;
    suspended_ms = currms - suspended_ms ;
    old_train_ms += suspended_ms ;
  }
}
